

-- BURST CACHE ---------------------------------------------------

local table = table;
local floor = floor;

local VUHDO_CONFIG;
local VUHDO_PANEL_SETUP;
local VUHDO_RAID;

local VUHDO_isTableHeadersShowing;
local VUHDO_isTableFootersShowing;
local VUHDO_getHealButtonWidth;
local VUHDO_getHeader;
local VUHDO_getHeaderPos;
local VUHDO_customizeHeader;
local VUHDO_getDynamicModelArray;
local VUHDO_getGroupMembersSorted;
local VUHDO_getHealButton;
local VUHDO_getHealButtonPos;
local VUHDO_setupAllHealButtonAttributes;
local VUHDO_isDifferentButtonPoint;
local VUHDO_addUnitButton;
local VUHDO_updateTargetBars;
local VUHDO_getHealthBar;
local VUHDO_getTargetButton;
local VUHDO_getTotButton;
local VUHDO_getOrCreateHealButton;
local VUHDO_reloadRaidMembers;
local VUHDO_updateAllCustomDebuffs;

function VUHDO_panelRefreshInitBurst()
  VUHDO_CONFIG = VUHDO_GLOBAL["VUHDO_CONFIG"];
  VUHDO_PANEL_SETUP = VUHDO_GLOBAL["VUHDO_PANEL_SETUP"];
  VUHDO_RAID = VUHDO_GLOBAL["VUHDO_RAID"];

  VUHDO_isTableHeadersShowing = VUHDO_GLOBAL["VUHDO_isTableHeadersShowing"];
  VUHDO_isTableFootersShowing = VUHDO_GLOBAL["VUHDO_isTableFootersShowing"];
  VUHDO_getHealButtonWidth = VUHDO_GLOBAL["VUHDO_getHealButtonWidth"];
  VUHDO_getHeader = VUHDO_GLOBAL["VUHDO_getHeader"];
  VUHDO_getHeaderPos = VUHDO_GLOBAL["VUHDO_getHeaderPos"];
  VUHDO_customizeHeader = VUHDO_GLOBAL["VUHDO_customizeHeader"];
  VUHDO_getDynamicModelArray = VUHDO_GLOBAL["VUHDO_getDynamicModelArray"];
  VUHDO_getGroupMembersSorted = VUHDO_GLOBAL["VUHDO_getGroupMembersSorted"];
  VUHDO_getHealButton = VUHDO_GLOBAL["VUHDO_getHealButton"];
  VUHDO_getHealButtonPos = VUHDO_GLOBAL["VUHDO_getHealButtonPos"];
  VUHDO_setupAllHealButtonAttributes = VUHDO_GLOBAL["VUHDO_setupAllHealButtonAttributes"];
  VUHDO_isDifferentButtonPoint = VUHDO_GLOBAL["VUHDO_isDifferentButtonPoint"];
  VUHDO_addUnitButton = VUHDO_GLOBAL["VUHDO_addUnitButton"];
  VUHDO_updateTargetBars = VUHDO_GLOBAL["VUHDO_updateTargetBars"];
  VUHDO_getHealthBar = VUHDO_GLOBAL["VUHDO_getHealthBar"];
	VUHDO_getTargetButton = VUHDO_GLOBAL["VUHDO_getTargetButton"];
	VUHDO_getTotButton = VUHDO_GLOBAL["VUHDO_getTotButton"];
	VUHDO_getOrCreateHealButton = VUHDO_GLOBAL["VUHDO_getOrCreateHealButton"];
	VUHDO_reloadRaidMembers = VUHDO_GLOBAL["VUHDO_reloadRaidMembers"];
	VUHDO_updateAllCustomDebuffs = VUHDO_GLOBAL["VUHDO_updateAllCustomDebuffs"];
end
-- BURST CACHE ---------------------------------------------------



--
local tModels;
local function VUHDO_hasPanelButtons(aPanelNum)
	if (not VUHDO_CONFIG["SHOW_PANELS"] or not VUHDO_IS_SHOWN_BY_GROUP) then
		return false;
	end

	tModels = VUHDO_PANEL_DYN_MODELS[aPanelNum];
	return tModels ~= nil and #tModels > 0;
end



--
local tCnt;
local tHeader;
local tX, tY;
local tBarScaling;
local tModel;
local tAnzCols;
local tWidth;

local function VUHDO_refreshPositionTableHeaders(aPanel, aPanelNum)
  tBarScaling = VUHDO_PANEL_SETUP[aPanelNum]["SCALING"];
	tModel = VUHDO_PANEL_DYN_MODELS[aPanelNum];
	tWidth = VUHDO_getHeaderWidth(aPanelNum);

	if (VUHDO_isTableHeadersShowing(aPanelNum) or VUHDO_isTableFootersShowing(aPanelNum)) then
		tAnzCols = #tModel;

		if (tAnzCols > VUHDO_MAX_GROUPS_PER_PANEL) then
			tAnzCols = VUHDO_MAX_GROUPS_PER_PANEL;
		end
	else
		tAnzCols = 0;
	end

	for tCnt = 1, tAnzCols do
		tHeader = VUHDO_getHeader(tCnt, aPanelNum);
		tX, tY = VUHDO_getHeaderPos(tCnt, aPanelNum);
		tX = floor(tX + tWidth / 2 * (1 - (tBarScaling.headerWidth / 100)));

	  if (VUHDO_isDifferentButtonPoint(tHeader, tX, -tY)) then
			tHeader:ClearAllPoints();
    	tHeader:SetPoint("TOPLEFT", aPanel:GetName(), "TOPLEFT", tX, -tY);
    end

    VUHDO_customizeHeader(tHeader, aPanelNum, tModel[tCnt]);
		tHeader:Show();
	end

	for tCnt = tAnzCols + 1, VUHDO_MAX_GROUPS_PER_PANEL do
		VUHDO_getHeader(tCnt, aPanelNum):Hide();
	end
end



--
local tModelIndex;
local tIsVehicle;
local tCnt;
local tGroupIdx;
local tColIdx;
local tButtonIdx;
local tModels;
local tSortBy;
local tPanelName;
local tManaHeight, tManaBaseHeight, tHealBaseHeight;
local tInfo;
local tIsShowRage;
local tIsShowMana;
local tSetup, tScaling;
local tX, tY;
local tButton;
local tUnit;
local tGroupArray;
local tModelId;
local function VUHDO_refreshPositionAllHealButtons(aPanel, aPanelNum)
	tSetup = VUHDO_PANEL_SETUP[aPanelNum];
	tModels = VUHDO_getDynamicModelArray(aPanelNum);
  tSortBy = tSetup["MODEL"].sort;
  tPanelName = aPanel:GetName();

	tColIdx = 1;
	tButtonIdx = 1;

	tScaling = tSetup["SCALING"];
	tIsShowRage = tScaling.showRageBars;
	tIsShowMana = tScaling.showManaBars;
	tManaBaseHeight = tScaling.manaBarHeight;
	tHealBaseHeight = tScaling.barHeight;

  for tModelIndex, tModelId in ipairs(tModels) do
  	tGroupArray = VUHDO_getGroupMembersSorted(tModelId, tSortBy, aPanelNum, tModelIndex);
  	tIsVehicle = VUHDO_ID_VEHICLES == tModelId;

  	tGroupIdx = 1;
  	for _, tUnit in ipairs(tGroupArray) do

	  	tButton = VUHDO_getOrCreateHealButton(tButtonIdx, aPanelNum);
	  	tButtonIdx = tButtonIdx + 1;
	  	if (tButtonIdx > VUHDO_MAX_BUTTONS_PANEL) then
	  		return;
	  	end

	  	tX, tY = VUHDO_getHealButtonPos(tColIdx, tGroupIdx, aPanelNum);
	  	if (VUHDO_isDifferentButtonPoint(tButton, tX, -tY)) then
  			tButton:ClearAllPoints();
      	tButton:SetPoint("TOPLEFT", tPanelName, "TOPLEFT", tX, -tY);
      end
			VUHDO_setupAllHealButtonAttributes(tButton, tUnit, false, tIsVehicle, false);
      VUHDO_setupAllTargetButtonAttributes(VUHDO_getTargetButton(tButton), tUnit);
			VUHDO_setupAllTotButtonAttributes(VUHDO_getTotButton(tButton), tUnit);
	  	VUHDO_addUnitButton(tButton);
      tButton:Show();

      VUHDO_updateTargetBars(tUnit);

		  tInfo = VUHDO_RAID[tUnit];
		  if (tInfo == nil or ((tInfo["powertype"] == 0 or tIsShowRage) and tIsShowMana)) then
  			tManaHeight = tManaBaseHeight;
  		else
  			tManaHeight = 0;
  		end

			VUHDO_getHealthBar(tButton, 1):SetHeight(tHealBaseHeight - tManaHeight)
			VUHDO_getHealthBar(tButton, 2):SetHeight(tManaHeight);

  		tGroupIdx = tGroupIdx + 1;
  	end

  	tColIdx = tColIdx + 1;
  end

  for tCnt = tButtonIdx, VUHDO_MAX_BUTTONS_PANEL do
		tButton = VUHDO_getHealButton(tCnt, aPanelNum);
		if (tButton == nil) then
			break;
		end
		tButton.raidid = nil;
		tButton.target = nil;
		tButton:SetAttribute("unit", nil);
		tButton:ClearAllPoints();
		tButton:Hide();
  end
end



--
local tHeight;
local function VUHDO_refreshInitPanel(aPanel, aPanelNum)
	tHeight = VUHDO_getHealPanelHeight(aPanelNum);
	if (tHeight < 30) then
		tHeight = 30;
	end
	aPanel:SetHeight(tHeight);
	aPanel:SetWidth(VUHDO_getHealPanelWidth(aPanelNum));
end



--
local tPanel;
local function VUHDO_refreshPanel(aPanelNum)

  if (VUHDO_hasPanelButtons(aPanelNum)) then
		tPanel = VUHDO_getActionPanel(aPanelNum);
	  tPanel:Show();

		VUHDO_refreshInitPanel(tPanel, aPanelNum);
		VUHDO_refreshPositionTableHeaders(tPanel, aPanelNum);
	end

	-- Even if model is not in panel, we need to refresh VUHDO_UNIT_BUTTONS
	if (VUHDO_isPanelPopulated(aPanelNum)) then
		tPanel = VUHDO_getActionPanel(aPanelNum);
		VUHDO_refreshPositionAllHealButtons(tPanel, aPanelNum);
	end
end



--
local tCnt;
local tPanel;
local function VUHDO_refreshAllPanels()
	for tCnt = 1, VUHDO_MAX_PANELS do
		if (VUHDO_isPanelVisible(tCnt)) then
			VUHDO_refreshPanel(tCnt);
		else
			VUHDO_getActionPanel(tCnt):Hide();
		end
	end

	VUHDO_updateAllRaidBars();
	VuhDoGcdStatusBar:Hide();
end



--
function VUHDO_refreshUI()
	VUHDO_IS_RELOADING = true;
	VUHDO_reloadRaidMembers();
	table.wipe(VUHDO_UNIT_BUTTONS);
	VUHDO_refreshAllPanels();
	VUHDO_updateAllCustomDebuffs(true);
	VUHDO_IS_RELOADING = false;
end
