local strlen = strlen;
local strsub = strsub;
local InCombatLockdown = InCombatLockdown;
local table = table;
local tinsert = tinsert;
local strfind = strfind;
local strsplit = strsplit;
local strbyte = strbyte;
local tonumber = tonumber;
local UnitInRaid = UnitInRaid;
local UnitExists = UnitExists;
local UnitIsUnit = UnitIsUnit;
local UnitInRange = UnitInRange;
local GetRaidRosterInfo = GetRaidRosterInfo;
local IsInInstance = IsInInstance;
local IsSpellInRange = IsSpellInRange;



-- returns an array of numbers sequentially found in a string
local tCnt;
local tNumbers = { };
local tIndex;
local tStrlen;
local tDigit;
local tIsInNumber;
function VUHDO_getNumbersFromString(aName, aMaxAnz)
	table.wipe(tNumbers);
	tIndex = 0;
	tStrlen = strlen(aName);
	tIsInNumber = false;

	for tCnt = 1, tStrlen do
		tDigit = strbyte(aName, tCnt) - 48;
		if (tDigit >= 0 and tDigit <= 9) then
			if (tIsInNumber) then
				tNumbers[tIndex] = tNumbers[tIndex] * 10 + tDigit;
			else
				tIsInNumber = true;
				tIndex = tIndex + 1;
				tNumbers[tIndex] = tDigit;
			end
		else
			if (tIndex >= aMaxAnz) then
				return tNumbers;
			end

			tIsInNumber = false;
		end
	end

	return tNumbers;
end



--
local tValue;
function VUHDO_tableUniqueAdd(aTable, aValue)
	for _, tValue in pairs(aTable) do
		if (tValue == aValue) then
			return false;
		end
	end

	tinsert(aTable, aValue);
	return true;
end



local VUHDO_RAID_NAMES;
local VUHDO_RAID;
local VUHDO_UNIT_BUTTONS;
local VUHDO_CONFIG;
function VUHDO_toolboxInitBurst()
	VUHDO_RAID_NAMES = VUHDO_GLOBAL["VUHDO_RAID_NAMES"];
	VUHDO_RAID = VUHDO_GLOBAL["VUHDO_RAID"];
	VUHDO_UNIT_BUTTONS = VUHDO_GLOBAL["VUHDO_UNIT_BUTTONS"];
	VUHDO_CONFIG = VUHDO_GLOBAL["VUHDO_CONFIG"];
end



local VUHDO_PROFILE_TIMER = 0; -- Profilers starting time stamp



-- Init by setting start time stamp
function VUHDO_initProfiler()
	VUHDO_Msg("Init Profiler");
	VUHDO_PROFILE_TIMER = GetTime() * 1000;
end



-- Dump the duration in ms since profiler has been initialized
function VUHDO_seeProfiler()
	local tTimeDelta = floor(GetTime() * 1000 - VUHDO_PROFILE_TIMER);
	VUHDO_Msg("Duration: " .. tTimeDelta);
end



--
local tInfo;
function VUHDO_dumpRaid()
	for _, tInfo in pairs(VUHDO_RAID) do
		VUHDO_Msg(tInfo.name .. ": " .. tInfo.unit);
	end
end



-- Print chat frame line with no "{Vuhdo} prefix
function VUHDO_MsgC(aMessage, aRed, aGreen, aBlue)
	aRed = aRed or 1;
	aGreen = aGreen or 0.7;
	aBlue = aBlue or 0.2;

	DEFAULT_CHAT_FRAME:AddMessage(aMessage, aRed, aGreen, aBlue);
end



-- Print a standard chat frame
function VUHDO_Msg(aMessage, aRed, aGreen, aBlue)
	if (aMessage == nil) then
		aMessage = "<nil>";
	elseif (type(aMessage) == "table") then
		aMessage = "<table>";
	elseif (type(aMessage) == "boolean") then
		if (aMessage) then
			aMessage = "<true>";
		else
			aMessage = "<false>";
		end
	elseif (aMessage == "") then
		aMessage = " ";
	end
	VUHDO_MsgC("|cffffe566{VuhDo}|r " .. aMessage, aRed, aGreen, aBlue)
end



-- Returns the type of a given model id
function VUHDO_getModelType(aModelId)
	if ((aModelId >= VUHDO_ID_GROUP_1 and aModelId <= VUHDO_ID_GROUP_8) or aModelId == VUHDO_ID_GROUP_OWN) then
		return VUHDO_ID_TYPE_GROUP;
	elseif (aModelId >= VUHDO_ID_WARRIORS and aModelId <= VUHDO_ID_DEATH_KNIGHT) then
		return VUHDO_ID_TYPE_CLASS;
	elseif (aModelId == VUHDO_ID_UNDEFINED) then
	  return VUHDO_ID_TYPE_UNDEFINED;
	else
		return VUHDO_ID_TYPE_SPECIAL;
	end
end



-- returns unit-prefix, pet-prefix and maximum number of players in a party
function VUHDO_getUnitIds()
	if (UnitInRaid("player")) then
		return "raid", "raidpet";
	-- UnitInParty() ist buggy
	elseif (UnitExists("party1")) then
		return "party", "partypet";
	else
		return "player", "pet";
	end
end
local VUHDO_getUnitIds = VUHDO_getUnitIds;



-- Extracts unit number from a Unit's name
local tUnitNo;
function VUHDO_getUnitNo(aUnit)
	tUnitNo = tonumber(strsub(aUnit, -2, -1));
	if (tUnitNo ~= nil) then
		return tUnitNo;
	end

	tUnitNo = tonumber(strsub(aUnit, -1));
	if (tUnitNo ~= nil) then
		return tUnitNo;
	end

	if ("focus" == aUnit) then
		return 0;
	end

	return 1;
end



-- returns the units subgroup number, or 0 for pets/focus
local tGroupNo;
function VUHDO_getUnitGroup(aUnit, anIsPet)
	if (anIsPet or aUnit == nil or aUnit == "focus") then
		return 0;
	elseif (UnitInRaid("player")) then
		_, _, tGroupNo, _, _, _, _, _ = GetRaidRosterInfo(VUHDO_getUnitNo(aUnit));
		return tGroupNo;
	else
		return 1;
	end
end



-- returns wether or not a unit is in range
function VUHDO_isInRange(aUnit)
	if (VUHDO_CONFIG.RANGE_CHECK) then
		if (VUHDO_CONFIG.RANGE_PESSIMISTIC) then
			return UnitInRange(aUnit);
		else
			return UnitInRange(aUnit) or (1 == IsSpellInRange(VUHDO_CONFIG.RANGE_SPELL, aUnit));
		end
	else
		return true;
	end
end



-- Parses a aString line into an array of arguments
function VUHDO_textParse(aString)
   local tTextLen = 1;
   local tOutStrings = {};
   local tTextNum = 1;
   local tStartIdx = 1;
   local tStopIdx = 1;
   local tTextIdxStart = 1;
   local tTextIdxStop = 1;
   local tTextLeft = 1;
   local tNextSpaceIdx = 1;
   local tTextChunk = "";
   local tAnzIterations = 1;
   local tIsErroneous = false;

   if ((aString ~= nil) and (aString ~= "")) then
      while (string.find(aString, "  ") ~= nil) do
     	  aString = string.gsub(aString, "  ", " ");
      end

      if (string.len(aString) <= 1) then
         tIsErroneous = true;
      end

      if tIsErroneous ~= true then
        tTextIdxStart = 1;
        tTextIdxStop = string.len(aString);

        if (string.sub(aString, tTextIdxStart, tTextIdxStart) == " ") then
           tTextIdxStart = tTextIdxStart+1;
        end

        if (string.sub(aString, tTextIdxStop, tTextIdxStop) == " ") then
           tTextIdxStop = tTextIdxStop-1;
        end

        aString = string.sub(aString, tTextIdxStart, tTextIdxStop);
      end

      tTextNum = 1;
      tTextLeft = string.len(aString);

      while (tStartIdx <= tTextLeft) and (tIsErroneous ~= true) do

         tNextSpaceIdx = string.find(aString, " ",tStartIdx);
         if (tNextSpaceIdx ~= nil) then
            tStopIdx = (tNextSpaceIdx - 1);
         else
            tStopIdx = string.len(aString);
            LetsEnd = true;
         end

         tTextChunk = string.sub(aString, tStartIdx, tStopIdx);
         tOutStrings[tTextNum] = tTextChunk;
         tTextNum = tTextNum + 1;

         tStartIdx = tStopIdx + 2;

      end
   else
      tOutStrings[1] = "Error: Bad value passed to TextParse!";
   end

   if (tIsErroneous ~= true) then
      return tOutStrings;
   else
      return {"Error: Bad value passed to TextParse!"};
   end
end



-- return wether or not player is in fight an thus certain secure unit
-- operations cannot be done.
function VUHDO_isInFight()
	return InCombatLockdown();
end



-- Returns a "deep" copy of a table,
-- which means containing tables will be copies value-wise, not by reference
function VUHDO_deepCopyTable(aTable)
	local tDestTable = { };
	local tKey, tValue;

	for tKey, tValue in pairs(aTable) do
		if ("table" == type(tValue)) then
			tDestTable[tKey] = VUHDO_deepCopyTable(tValue);
		else
			tDestTable[tKey] = tValue;
		end
	end

	return tDestTable;
end



-- Returns a "deep" copy of a table, not deleting existent elements in aDestTable
-- which means containing tables will be copies value-wise, not by reference
local tIsBool;
function VUHDO_deepCopyTableTo(aTable, aDestTable)
	local tKey, tValue;

	if (aDestTable == nil) then
		aDestTable = { };
	end

	for tKey, tValue in pairs(aTable) do
		if ("table" == type(tValue)) then
			aDestTable[tKey] = VUHDO_deepCopyTableTo(tValue, aDestTable[tKey]);
		else
			aDestTable[tKey] = tValue;
		end
	end

	for tKey, tValue in pairs(aDestTable) do
		tIsBool = "boolean" == type(tValue) or ("number" == type(tValue) and (tValue == 0 or tValue == 1));

		if (tIsBool and not aTable[tKey]) then
			aDestTable[tKey] = aTable[tKey];
		end
	end

	return aDestTable;
end



--
function VUHDO_countTableDiffs(aTable, anotherTable)
	local tCount = 0;
	local tKey, tValue;

	if (aTable == nil or anotherTable == nil) then
		return 0;
	end

	for tKey, tValue in pairs(aTable) do
		if ("table" == type(tValue)) then
			tCount = tCount + VUHDO_countTableDiffs(tValue, anotherTable[tKey]);
		else
			if (aTable[tKey] ~= anotherTable[tKey]) then
				tCount = tCount + 1;
			end
		end
	end

	for tKey, tValue in pairs(anotherTable) do
		if ("table" == type(tValue)) then
		else
			if (aTable[tKey] == nil and tValue ~= aTable[tKey]) then
				tCount = tCount + 1;
			end
		end
	end

	return tCount;
end



--
local tCount;
function VUHDO_tableCount(anArray)
  tCount = 0;
  for _, _ in pairs(anArray) do
    tCount = tCount + 1;
  end

  return tCount;
end



-- Tokenizes a String into an array of strings, which were delimited by "aChar"
local tFrags = { };
function VUHDO_splitString(aText, aChar)
	table.wipe(tFrags);
	tFrags = { strsplit(aChar, aText) };
	return tFrags;
end



-- returns true if player currently is in a battleground
local tType;
local function VUHDO_isInBattleground()
	_, tType = IsInInstance();
	return "pvp" == tType or "arena" == tType;
end



-- returns the appropriate addon message channel for player
function VUHDO_getAddOnDistribution()
	if (VUHDO_isInBattleground()) then
		return "BATTLEGROUND";
	elseif(UnitInRaid("player")) then
		return "RAID";
	else
		return "PARTY";
	end
end



-- returns the players rank in a raid which is 0 = raid member, 1 = assist, 2 = leader
-- returns leader if not in raid, and member if solo, as no main tank are needed
local tUnitNo;
local tRank, tIsMl;
function VUHDO_getPlayerRank()
	for tUnit, _ in pairs(VUHDO_RAID) do
		if (UnitIsUnit(tUnit, "player")) then
			if (UnitInRaid("player")) then
				tUnitNo = VUHDO_getUnitNo(tUnit);
				_, tRank, _, _, _, _, _, _, _, _, tIsMl = GetRaidRosterInfo(tUnitNo);
				return tRank, tIsMl;
			elseif (GetNumPartyMembers() > 0) then
				return 2, true;
			else
				return 2, true;
			end
		end
	end

	return 2, true;
end


-- returns the units rank in a raid which is 0 = raid member, 1 = assist, 2 = leader
-- returns 2 if not in raid
local tRank, tIsMl;
function VUHDO_getUnitRank(aUnit)
	if (UnitInRaid("player")) then
		_, tRank, _, _, _, _, _, _, _, _, tIsMl = GetRaidRosterInfo(VUHDO_getUnitNo(aUnit));
		return tRank, tIsMl;
	end

	return 2, true;
end



-- returns the raid unit of player eg. "raid13" or "party4"
function VUHDO_getPlayerRaidUnit()
	for tUnit, _ in pairs(VUHDO_RAID) do
		if (UnitIsUnit("player", tUnit)) then
			return tUnit;
		end
	end

	return "player";
end
local VUHDO_getPlayerRaidUnit = VUHDO_getPlayerRaidUnit;



--
function VUHDO_getVersion(aVersionAwareArray)
	if (aVersionAwareArray ~= nil) then
		if (aVersionAwareArray["VERSION"] == nil) then
			return -1;
		else
			return aVersionAwareArray["VERSION"];
		end
	else
		return -1;
	end
end



--
function VUHDO_getNumGroupMembers(aGroupId)
	if (VUHDO_GROUPS[aGroupId] == nil) then
		return 0;
	end
	return #VUHDO_GROUPS[aGroupId];
end



--
local tZoneOkay;
local tUnitZone, tPlayerZone;
local tIdx;
function VUHDO_isInSameZone(aUnit)
	if (UnitIsUnit(aUnit, "player")) then
		return true;
	end

	tZoneOkay = true;
	if (UnitInRaid("player") and UnitExists(aUnit)) then

  	tIdx = VUHDO_getUnitNo(aUnit);
		_, _, _, _, _, _, tUnitZone, _, _ = GetRaidRosterInfo(tIdx);

		if (VUHDO_PLAYER_RAID_ID ~= nil) then
			tIdx = VUHDO_getUnitNo(VUHDO_PLAYER_RAID_ID);
			_, _, _, _, _, _, tPlayerZone, _, _ = GetRaidRosterInfo(tIdx);
			if (tUnitZone ~= tPlayerZone) then
				tZoneOkay = false;
			end
		end
	end

	return tZoneOkay;
end



-- Returns health of unit info in Percent
function VUHDO_getUnitHealthPercent(anInfo)
	if (anInfo.healthmax == 0) then
		return 0;
	end

	if (anInfo.health < anInfo.healthmax) then
		return (anInfo.health / anInfo.healthmax) * 100;
	else
		return 100;
	end
end



--
function VUHDO_isSpellKnown(aSpellName)
	return GetSpellInfo(aSpellName) ~= nil;
end



--
local tDeltaSecs;
function VUHDO_getDurationTextSince(aStartTime)
	if (aStartTime == nil) then
		return "";
	end

	tDeltaSecs = GetTime() - aStartTime;

	if (tDeltaSecs >= 3600) then
		return "(|cffffffff" .. floor(tDeltaSecs / 360) * 10 .. " hours|r)";
	elseif(tDeltaSecs >= 60) then
		return "(|cffffffff" .. floor(tDeltaSecs / 60) .. " mins|r)";
	else
		return "(|cffffffff" .. floor(tDeltaSecs) .. " secs|r)";
	end
end



--
local tPet;
function VUHDO_getPetUnit(anOwnerUnit)
	_, tPet, _ = VUHDO_getUnitIds();

	if ("focus" == anOwnerUnit) then
		return nil;
	elseif ("raidpet" == tPet) then
		if ("player" == anOwnerUnit) then
			anOwnerUnit = VUHDO_getPlayerRaidUnit(); -- Too early for VUHDO_PLAYER_RAID_ID

			if (anOwnerUnit == nil) then
				return nil;
			end
		end
		return tPet .. VUHDO_getUnitNo(anOwnerUnit);
	elseif ("partypet" == tPet) then
		if (anOwnerUnit == "player") then
			return "playerpet";
		end

		return tPet .. VUHDO_getUnitNo(anOwnerUnit);
	else
		return "pet";
	end
end



--
function VUHDO_getTargetUnit(aSourceUnit)
	if (UnitIsUnit("player", aSourceUnit)) then
		return "target";
	else
		return aSourceUnit .. "target";
	end
end



--
function VUHDO_getResurrectionSpells()
	if (VUHDO_RESURRECTION_SPELLS[VUHDO_PLAYER_CLASS] == nil) then
		return nil, nil;
	elseif (#VUHDO_RESURRECTION_SPELLS[VUHDO_PLAYER_CLASS] == 1) then
		return VUHDO_RESURRECTION_SPELLS[VUHDO_PLAYER_CLASS][1], nil;
	else
		if (VUHDO_isInFight()) then
			return VUHDO_RESURRECTION_SPELLS[VUHDO_PLAYER_CLASS][2], VUHDO_RESURRECTION_SPELLS[VUHDO_PLAYER_CLASS][1];
		else
			return VUHDO_RESURRECTION_SPELLS[VUHDO_PLAYER_CLASS][1], VUHDO_RESURRECTION_SPELLS[VUHDO_PLAYER_CLASS][2];
		end
	end
end



--
local tBtnUnit;
local tBtnName;
function VUHDO_resolveButtonUnit(aButton)
	tBtnUnit = aButton:GetAttribute("unit");
	tBtnName = VUHDO_RAID_NAMES[tBtnUnit];
	if (tBtnName ~= nil and "focus" ~= tBtnUnit) then
		return tBtnName;
	else
		return tBtnUnit;
	end
end



--
local tBtnInfo;
local tButtons;
function VUHDO_getUnitButtons(aUnit)
	tBtnInfo = VUHDO_RAID[aUnit];
	if (tBtnInfo ~= nil and "focus" ~= aUnit) then
		tButtons = VUHDO_UNIT_BUTTONS[tBtnInfo.name];
		if (tButtons ~= nil) then
			return tButtons;
		end
	end

	return VUHDO_UNIT_BUTTONS[aUnit];
end



--
local tVariantMax = nil;
local tBuffInfo;
function VUHDO_getBuffVariantMaxTarget(someVariants)

	for _, tBuffInfo in pairs(someVariants) do
		if (VUHDO_GROUPS_BUFFS[tBuffInfo[2]] ~= nil) then
			return tBuffInfo;
		else
			tVariantMax = tBuffInfo;
		end
	end

	return tVariantMax;
end



--
local tBuff, tBuffInfo;
function VUHDO_getBuffVariantSingleTarget(someVariants)

	for _, tBuffInfo in pairs(someVariants) do
		if (VUHDO_GROUPS_BUFFS[tBuffInfo[2]] == nil) then
			return tBuffInfo;
		else
			tBuff = tBuffInfo;
		end
	end

	return tBuff;
end
